package fr.llibre.ml2;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

/**
 * Classe permettant de calculer les coordonnées horizontales des planètes (pas Pluton)
 * ainsi que celle du Soleil
 * @author llibre
 */

public class Planet {

	// Magnitudes apparentes pifométrées pour les planètes */
	public static final double moys[] = {1.7, -4.2, -26.7, 0.55, -2.2, 0.48, 5.65, 7.85};
	private static final double amps[] = {3.8, 0.4, 0, 2.35, 0.6, 0.72,0.35, 0.15};
	private static final double UA = 149597870691.; // unité astronomique
	private static final double vlum_uaj = 299792458.*86400./UA; // vitesse lumière en UA/jour

	// Paramètres de l'instance
	public int numPla = -1;
	public String nom;
	public double jeCou = -1.;
	public double magpla;   // Magnitude pifométrée
	public double lambG_rd; // Longitude écliptique géocentrique
	public double betaG_rd; // Latitude écliptique géocentrique
	public double lambH_rd; // Longitude écliptique héliocentrique
	public double betaH_rd; // Latitude écliptique héliocentrique
	public double rho_ua;   // Distance à la Terre
	public double ascDr_rd; // Ascension droite apparente
	public double decli_rd; // Déclinaison apparente
	double hourAng_rd;
	double hourAng_dg;
	double azi_rd ;
	double haut_rd;
	double hautCor_dg;

	public Planet(int noPla)
	{
		if (noPla >= 0 && noPla < 8) 
		{
			numPla = noPla;
			if (numPla == 2) nom = "Soleil";
			else nom = Astro.nomPla[numPla];;
		}
	}

	public Planet(String nomPla)
	{
		String name = Utils.sansAccents(nomPla);
		for (int n = 0; n <  8; n++)
			if(name.equalsIgnoreCase(Utils.sansAccents(Astro.nomPla[n]))) numPla = n;
		if(numPla >= 0)
		{
			if (numPla == 2) nom = "Soleil";
			else nom = Astro.nomPla[numPla];;
		}
	}

	/**
	 *    Mise à jour des coordonnées de la planète (ou du Soleil) par rapport à la Terre
	 * @param ju       : date UT en jour julien
	 * @param longi_dg : longitude du lieu en degrés
	 * @param lati_dg  : latitude du lieu en degrés
	 * @param sexaDec  : >0 sexagésimal, <0 décimal, 0 pas d'impression
	 * @param VSOP87   : true => calcul par VSOP87, sinon par paramètres orbitaux classiques
	 * @param eco      : true => utilisation des paramètres astro variant lentement
	 */
	public String infos(double ju, double longi_dg, double lati_dg, int sexaDec, boolean VSOP87, boolean eco)
	{
		ju2ecliequa_rd(ju, VSOP87, eco);			
		double lambG_dg = Math.toDegrees(lambG_rd);			
		double betaG_dg = Math.toDegrees(betaG_rd);
		double lambH_dg = Math.toDegrees(lambH_rd);			
		double betaH_dg = Math.toDegrees(betaH_rd);
		double ascDr_dg = Math.toDegrees(ascDr_rd);			
		double decli_dg = Math.toDegrees(decli_rd);
		double longi_rd = Math.toRadians(longi_dg);
		double lati_rd = Math.toRadians(lati_dg);		
		double ttopo[] = new double[3];
		hourAng_rd = Astro.equa2topo_rd(ju, ascDr_rd, decli_rd, longi_rd, lati_rd, ttopo, eco);
		hourAng_dg = Math.toDegrees(hourAng_rd);			
		azi_rd = ttopo[0];
		double azi_dg = Math.toDegrees(azi_rd);
		haut_rd = ttopo[1];
		double haut_dg = Math.toDegrees(haut_rd);
		hautCor_dg = ttopo[2];

		String str = "";
		if (sexaDec != 0)
		{
			DecimalFormatSymbols dfs = new DecimalFormatSymbols(); dfs.setDecimalSeparator('.');	
			DecimalFormat df1 = new DecimalFormat("0.0", dfs);			
			DecimalFormat df4 = new DecimalFormat("0.0000", dfs);			
			str += "Magnitude app. (approx.) " + df1.format(magpla) + "\n";
			if (sexaDec > 0)
			{
				str += "C. eclip. Helio.: " + Utils.deg2dms(lambH_dg)
						+ " | " + Utils.deg2dms2(betaH_dg) + "\n";
				str += "C. eclip. Geoce.: " + Utils.deg2dms(lambG_dg)
						+ " | " + Utils.deg2dms2(betaG_dg) + "\n";
				if (numPla != 2) str += "Distance " + df4.format(rho_ua) + " U.A.\n";
				str += "C. équat.       : " + Utils.deg2hms(ascDr_dg)
						+ " | " + Utils.deg2dms2(decli_dg) + "\n";
				str += "C. hor. app.: " + Utils.deg2hms(hourAng_dg) + " (" + Utils.deg2dms(hourAng_dg)
						+ ") | " + Utils.deg2dms2(decli_dg) + "\n";
				str += "C. horizontales : " + Utils.deg2dms2(azi_dg)
						+ " | " + Utils.deg2dms2(haut_dg) + "\n";
				str += "Hauteur corrigée: " + Utils.deg2dms2(hautCor_dg) + "\n";	
			}
			else
			{				
				str += "C. eclip. Hel.  : " + df4.format(lambH_dg)
						+ "° | " + df4.format(betaH_dg) + "°\n";
				str += "C. eclip. Geo.  : " + df4.format(lambG_dg)
						+ "° | " + df4.format(betaG_dg) + "°\n";
				if (numPla != 2) str += "Distance " + df4.format(rho_ua) + " U.A.\n";
				str += "C. équat.       : " + df4.format(ascDr_dg/15)
						+ "h | " + df4.format(decli_dg) + "°\n";
				str += "C. hor. app.: " + df4.format(hourAng_dg/15.) + "h (" + df4.format(hourAng_dg)
						+ "°) | " + df4.format(decli_dg) + "°\n";
				str += "C. horizontales : " + df4.format(azi_dg)
						+ "° | " + df4.format(haut_dg) + "°\n";
				str += "Hauteur corrigée: " + df4.format(hautCor_dg) + "°\n";
			}
			// CALCUL TRANSIT
			double [] result = new double [7];
			boolean interOk = Transit.getResult(this, ju, longi_dg, lati_dg, eco, result);
			if (sexaDec != 0) str +=  Transit.getString(result, sexaDec, interOk);
		}
		return str;
	}

	private void position(double je, int nopla, double xyzlbr[], boolean VSOP87)
	{

		if(VSOP87) PlanetVsop87.position(je, nopla, xyzlbr);
		else PlanetEop.position(je, nopla, xyzlbr);	
	}	

	/**
	 *  Calcul coordonnées ecliptiques et équatoriales de planètes
	 * @param ju : jour julien TU civil
	 * @param VSOP87 : si true calcul par VSOP87, sinon par paramètres orbitaux classiques
	 * @param eco    : si true utilisation des paramètres astro variant lentement
	 *  Les variables suivantes sont mises à jour :
	 *  ascDr_rd : Ascension droite en radians
	 *  decli_rd : Déclinaison en radians	
	 *  rho_ua   : Distance à la Terre en UA
	 *  magpla   : magnitude pifométrée
	 *	lambG_rd : Longitude géocentrique en radians
	 *  betaG_rd : Latitude géocentrique en radians
	 *  lambH_rd : Longitude héliocentrique en radians		
	 *  betaH_rd : latitude héliocentrique en radians	  
	 *  Remarque : Dans le cas du Soleil, les 2 derniers paramètres contiennent les longitude et
	 *  latitude écliptique héliocentriques de la Terre, sans corection de nutation. A ne pas utiliser.
	 */
	void ju2ecliequa_rd(double ju, boolean VSOP87, boolean eco)
	{
		double xyzlbrT[] = new double[6];
		double xyzlbrP[] = new double[6];
		double [] temp = {0};
		double deltaT;
		if(eco) deltaT = Astro.deltaT;	
		else deltaT =  Utils.jul2deltaT(ju);
		double je = ju + deltaT;				
		if (numPla == 2)
		{// Cas position Soleil remplace Terre 
			position(je, 2, xyzlbrP, VSOP87);  // Pos Terre/Sol
			double rho1 = xyzlbrP[5];
			// Correction vitesse lumiere
			double tau = rho1/vlum_uaj;
			// Position quand la lumière est partie
			position(je-tau, 2, xyzlbrP, VSOP87);
			rho_ua = xyzlbrP[5];
			lambH_rd = xyzlbrP[3]; // longi Helio de la terre
			betaH_rd = xyzlbrP[4]; // lati helio de la terre			
			// Géocentriques du Soleil : coordonnées opposées
			if (xyzlbrP[3] < Math.PI) lambG_rd = xyzlbrP[3] + Math.PI;
			else betaG_rd = xyzlbrP[3] - Math.PI;
			betaG_rd = -xyzlbrP[4];
		}
		else 
		{// cas position planètes / Soleil
			position(je, 2, xyzlbrT, VSOP87); 
			position(je, numPla, xyzlbrP, VSOP87); 
			// Position planete / Terre
			double rho1 = 0.;
			for(int i = 0; i< 3; i++) 
			{
				xyzlbrP[i] -= xyzlbrT[i]; 
				rho1 += xyzlbrP[i]*xyzlbrP[i];
			}
			rho1 = Math.sqrt(rho1);
			double tau = rho1/vlum_uaj; 
			boolean combine = true;
			if (combine)
			{// Corrections temps-lumière et aberration combinés			 
				position(je-tau, 2, xyzlbrT, VSOP87); 
				position(je-tau, numPla, xyzlbrP, VSOP87); 
				lambH_rd = xyzlbrP[3]; // longi Helio
				betaH_rd = xyzlbrP[4]; // lati helio							
				// Position planete / Terre
				rho_ua = 0.;
				for(int i = 0; i < 3; i++) 
				{
					xyzlbrP[i] -= xyzlbrT[i];  
					rho_ua += xyzlbrP[i]*xyzlbrP[i];
				}
				rho_ua = Math.sqrt(rho_ua);
				lambG_rd = Math.atan2(xyzlbrP[1],xyzlbrP[0]);
				betaG_rd = Math.asin(xyzlbrP[2]/rho_ua); 
			}		
			else
			{// Correction temp-lumière à part		
				position(je-tau, numPla, xyzlbrP, VSOP87); 
				lambH_rd = xyzlbrP[3]; // longi Helio
				betaH_rd = xyzlbrP[4]; // lati helio			
				rho_ua = 0.;
				for(int i = 0; i< 3; i++) 
				{
					xyzlbrP[i] -= xyzlbrT[i];  
					rho_ua += xyzlbrP[i]*xyzlbrP[i];
				}
				rho_ua = Math.sqrt(rho_ua);
				lambG_rd = Math.atan2(xyzlbrP[1],xyzlbrP[0]);
				betaG_rd = Math.asin(xyzlbrP[2]/rho_ua); 
				// Correction aberration à part
				if (eco) lambG_rd += Astro.aberrEcli_rd_eco(je - Astro.deltaT, lambG_rd, betaG_rd, temp);
				else
				{
					double S = (je - 2451545)/36525.; // siecle julien
					lambG_rd += Astro.aberrEcli_rd_std(lambG_rd, betaG_rd, 
							Math.toRadians(Astro.sun_longi_dg(S)),
							Astro.eccentricity(S), 
							Math.toRadians(Astro.earth_longiPerih_dg(S)), temp); 
				}
				betaG_rd += temp[0];
			}	
		}// fin cas planète		
		// Correction nutation en coordonnées ecliptiques
		double obli_rd;
		if (eco)
		{
			lambG_rd += Astro.dpsi_rd;
			obli_rd = Astro.obli_rd;
		}
		else 
		{
			lambG_rd += Astro.nutationEcli_rd(je, true, temp);
			obli_rd = Math.toRadians(Astro.obliquity_dg(je)) + temp[0];
		}			
		// Chamgement de repère Ecliptique => Equatorial		
		ascDr_rd = Astro.ecli2equa_rd(lambG_rd, betaG_rd, obli_rd, temp);
		decli_rd = temp[0];

		/* Magnitude pifométrées des 6 premières planètes
		 * Pour les planètes inférieures : maximale qd proche de la Terre
		 * et minimale qd de l'autre coté du Soleil
		 * Pour les planètes supérieures : maximales en quadrature et minimales à l'alignement */
		if (numPla != 2)
		{
			double cosalf = Math.cos(xyzlbrT[3]-xyzlbrP[3]);
			if (numPla < 2)  magpla = moys[numPla] - amps[numPla]*cosalf; // planètes inférieures
			else {magpla = moys[numPla] + amps[numPla]*(2*cosalf*cosalf -1.); } // Planètes supérieures
		}
		else magpla = -26.7; // magnitude soleil			
		return;
	}

	public static void main(String[] args) {
		double ju = Utils.dat2jul(2017,4,12,12,0,0.);
		for (int i = 0; i < 8; i++)
		{
			Planet p = new Planet(i);
			System.out.println("\nPosition de : " + p.nom);
			System.out.println(p.infos(ju, 0., 45., 1, true, true));
		}
	}		
}
